searchData={"items":[{"type":"behaviour","title":"ddskerl","doc":"DDSketch behaviour.","ref":"ddskerl.html"},{"type":"type","title":"ddskerl.ddsketch/0","doc":"DDSketch instance.","ref":"ddskerl.html#t:ddsketch/0"},{"type":"callback","title":"ddskerl.insert/2","doc":"Insert a value into the DDSketch.","ref":"ddskerl.html#c:insert/2"},{"type":"callback","title":"ddskerl.merge/2","doc":"Merge two DDSketch instances.","ref":"ddskerl.html#c:merge/2"},{"type":"callback","title":"ddskerl.new/1","doc":"Create a new DDSketch instance.","ref":"ddskerl.html#c:new/1"},{"type":"type","title":"ddskerl.opts/0","doc":"Options for the DDSketch.","ref":"ddskerl.html#t:opts/0"},{"type":"callback","title":"ddskerl.quantile/2","doc":"Calculate the quantile of a DDSketch.","ref":"ddskerl.html#c:quantile/2"},{"type":"callback","title":"ddskerl.sum/1","doc":"Get the sum number of elements in the DDSketch.","ref":"ddskerl.html#c:sum/1"},{"type":"callback","title":"ddskerl.total/1","doc":"Get the total number of elements in the DDSketch.","ref":"ddskerl.html#c:total/1"},{"type":"module","title":"ddskerl_bound","doc":"DDSketch implementation in Erlang.\n\nThis implements an bounded bucket count, that is, on a degenerate case,\nmemory consumption won't grow but lower quantiles will lose accuracy as the summary saturates.\n\nWhen the number of buckets exceeds the limit,\nthe two smallest buckets are collapsed into one and the limit is kept.\n\nThe implementation uses a local and non-shared `m:gb_trees`, and the buckets are not preallocated,\nbut are rather created on demand, without bounds on their indexes.\n\nBecause the data structure cannot be accessed concurretly (share-nothing semantics),\na good strategy would be to keep many running in parallel, and running queries over a merge.\n\n> #### When to use {: .tip}\n> This is a good choice when the summary is tracked by a process\n> and updates the state through its mailbox.\n>\n> For example, the following could be used as a template:\n> ```\n> ...\n> -behaviour(gen_server).\n>\n> -spec start_link(ddskerl_bound:opts()) -> gen_server:start_ret().\nstart_link(Opts) ->\n>    gen_server:start_link(?MODULE, Opts, [{spawn_opt, [{message_queue_data, off_heap}]}]).\n>\n> -spec init(ddskerl_bound:opts()) -> {ok, ddskerl_bound:ddsketch()}.\n> init(Opts) ->\n>    {ok, ddskerl_bound:new(Opts)}.\n>\n> handle_call({get_quantile, Q}, _From, Sketch) ->\n>    {reply, ddskerl_bound:quantile(Sketch, Q), Sketch}.\n>\n> handle_cast({new_value, Value}, Sketch) ->\n>    {noreply, ddskerl_bound:insert(Sketch, Value)}.\n> ...\n> ```","ref":"ddskerl_bound.html"},{"type":"opaque","title":"ddskerl_bound.ddsketch/0","doc":"DDSketch instance.","ref":"ddskerl_bound.html#t:ddsketch/0"},{"type":"function","title":"ddskerl_bound.insert/2","doc":"Insert a value into the DDSketch.","ref":"ddskerl_bound.html#insert/2"},{"type":"function","title":"ddskerl_bound.merge/2","doc":"Merge two DDSketch instances.","ref":"ddskerl_bound.html#merge/2"},{"type":"function","title":"ddskerl_bound.new/1","doc":"Create a new DDSketch instance.","ref":"ddskerl_bound.html#new/1"},{"type":"type","title":"ddskerl_bound.opts/0","doc":"Options for the DDSketch.","ref":"ddskerl_bound.html#t:opts/0"},{"type":"function","title":"ddskerl_bound.quantile/2","doc":"Calculate the quantile of a DDSketch.","ref":"ddskerl_bound.html#quantile/2"},{"type":"function","title":"ddskerl_bound.sum/1","doc":"Get the sum of elements in the DDSketch.","ref":"ddskerl_bound.html#sum/1"},{"type":"function","title":"ddskerl_bound.total/1","doc":"Get the total number of elements in the DDSketch.","ref":"ddskerl_bound.html#total/1"},{"type":"module","title":"ddskerl_counters","doc":"DDSketch implementation in Erlang.\n\nThis implements an optimised pre-allocated bounded bucket count, that is, on a degenerate case,\nmemory consumption won't grow but quantiles might lose accuracy as values escape bounds.\n\nIn this optimisation, buckets are preallocated using `m:counters`.\nValues within `(0,1)` will be inserted into the underflow bucket;\nvalues above the expected limit will be inserted into the overflow bucket.\n\n> #### About rounding errors {: .warning}\n> Due to the nature of the implementation, this algorithm will `round/1` floats when keeping track\n> of the minimum and maximums, hence these values might incurr rounding of a single digit errors.\n>\n> All other quantiles, as well as integers, will be tracked with the precision the sketch was\n> configured with.","ref":"ddskerl_counters.html"},{"type":"module","title":"Calculating the bucket count - ddskerl_counters","doc":"If you have an expected highest value `M` and a given error `E`,\nyour ideal bucket count would be given by the formula:\n`ceil(math:log2(M) * (1.0 / math:log2((1 + E) / (1 - E))))`.\n\nFor example, if you measure microseconds and you expect no operation to take more than an hour:\n```erlang\n1> F = fun(M, E) -> ceil(math:log2(M) * (1.0 / math:log2((1 + E) / (1 - E)))) end.\n2> F(3_600_000_000, 0.01).\n1101\n```\n\nNote than for an expected error of 1% and 2184 buckets, we can fit values\nall the way as big to the biggest 64bits signed integer.\nIf we're measuring picoseconds, this would suffice to measure 107 days.\n\n> #### When to use {: .tip}\n> This is a good choice when it is kept in a persistent_term and it is globally shared.\n> Note however all the perks and pitfalls of using a persistent term before choosing one.\n>\n> For example:\n> ```erlang\n> persistent_term:put(my_summary, ddskerl_counters:new(Opts)).\n> ...\n> ddskerl_counters:insert(persistent_term:get(my_summary), Value).\n> ...\n> ddskerl_counters:quantile(persistent_term:get(my_summary), Q).\n> ```","ref":"ddskerl_counters.html#module-calculating-the-bucket-count"},{"type":"opaque","title":"ddskerl_counters.ddsketch/0","doc":"DDSketch instance","ref":"ddskerl_counters.html#t:ddsketch/0"},{"type":"function","title":"ddskerl_counters.insert/2","doc":"Insert a value into the DDSketch","ref":"ddskerl_counters.html#insert/2"},{"type":"function","title":"ddskerl_counters.merge/2","doc":"Merge two DDSketch instances","ref":"ddskerl_counters.html#merge/2"},{"type":"function","title":"ddskerl_counters.new/1","doc":"Create a new DDSketch instance","ref":"ddskerl_counters.html#new/1"},{"type":"type","title":"ddskerl_counters.opts/0","doc":"Options for the DDSketch","ref":"ddskerl_counters.html#t:opts/0"},{"type":"function","title":"ddskerl_counters.quantile/2","doc":"Calculate the quantile of a DDSketch","ref":"ddskerl_counters.html#quantile/2"},{"type":"function","title":"ddskerl_counters.reset/1","doc":"Reset the DDSketch values to zero","ref":"ddskerl_counters.html#reset/1"},{"type":"function","title":"ddskerl_counters.sum/1","doc":"Get the sum of elements in the DDSketch","ref":"ddskerl_counters.html#sum/1"},{"type":"function","title":"ddskerl_counters.total/1","doc":"Get the total number of elements in the DDSketch","ref":"ddskerl_counters.html#total/1"},{"type":"module","title":"ddskerl_ets","doc":"DDSketch implementation in Erlang.\n\nThis implements an optimised pre-allocated bounded bucket count, that is, on a degenerate case,\nmemory consumption won't grow but quantiles might lose accuracy as values escape bounds.\n\nIn this optimisation, buckets are preallocated using `m:ets` tables.\nValues within `(0,1)` will be inserted into the underflow bucket;\nvalues above the expected limit will be inserted into the overflow bucket.","ref":"ddskerl_ets.html"},{"type":"module","title":"Calculating the bucket size - ddskerl_ets","doc":"If you have an expected highest value `M` and a given error `E`,\nyour ideal bucket size would be given by the formula:\n`ceil(math:log2(M) * (1.0 / math:log2((1 + E) / (1 - E))))`.\n\nFor example, if you measure microseconds and you expect no operation to take more than an hour:\n```erlang\n1> F = fun(M, E) -> ceil(math:log2(M) * (1.0 / math:log2((1 + E) / (1 - E)))) end.\n2> F(3600000000, 0.01).\n1101\n```\n\nNote than for an expected error of 1% and 2184 buckets, we can fit values\nall the way as big to the biggest 64bits signed integer.\nIf we're measuring picoseconds, this would suffice to measure 107 days.\n\n> #### When to use {: .tip}\n> This is a good choice when shared ets tables are a more efficient way to ensure dynamicity\n> and potentially code upgrades.\n>\n> For example:\n> ```erlang\n> ddskerl_ets:new(#{ets_table => Table, name => Name, ...}).\n> ...\n> ddskerl_ets:insert(Table, Name, Value).\n> ...\n> ddskerl_ets:quantile(Table, Name, Q).\n> ```","ref":"ddskerl_ets.html#module-calculating-the-bucket-size"},{"type":"opaque","title":"ddskerl_ets.ddsketch/0","doc":"DDSketch instance.","ref":"ddskerl_ets.html#t:ddsketch/0"},{"type":"function","title":"ddskerl_ets.insert/2","doc":"Insert a value into the DDSketch.","ref":"ddskerl_ets.html#insert/2"},{"type":"function","title":"ddskerl_ets.insert/3","doc":"Insert a value into the DDSketch.","ref":"ddskerl_ets.html#insert/3"},{"type":"function","title":"ddskerl_ets.merge/2","doc":"Merge two DDSketch instances.","ref":"ddskerl_ets.html#merge/2"},{"type":"function","title":"ddskerl_ets.merge/4","doc":"Merge two DDSketch instances.","ref":"ddskerl_ets.html#merge/4"},{"type":"function","title":"ddskerl_ets.new/1","doc":"Create a new DDSketch instance.","ref":"ddskerl_ets.html#new/1"},{"type":"function","title":"ddskerl_ets.new/4","doc":"Create a new DDSketch instance.","ref":"ddskerl_ets.html#new/4"},{"type":"opaque","title":"ddskerl_ets.object/0","doc":"DDSketch tuple.","ref":"ddskerl_ets.html#t:object/0"},{"type":"type","title":"ddskerl_ets.opts/0","doc":"Options for the DDSketch.\n\n`ets_table` refers to the table weher the sketch is stored,\n`name` is the key to use when storing the sketch in the ets table.","ref":"ddskerl_ets.html#t:opts/0"},{"type":"function","title":"ddskerl_ets.quantile/2","doc":"Calculate the quantile of a DDSketch.","ref":"ddskerl_ets.html#quantile/2"},{"type":"function","title":"ddskerl_ets.quantile/3","doc":"Calculate the quantile of a DDSketch.","ref":"ddskerl_ets.html#quantile/3"},{"type":"function","title":"ddskerl_ets.reset/1","doc":"Reset the DDSketch values to zero","ref":"ddskerl_ets.html#reset/1"},{"type":"function","title":"ddskerl_ets.reset/2","doc":"Reset the DDSketch values to zero","ref":"ddskerl_ets.html#reset/2"},{"type":"function","title":"ddskerl_ets.sum/1","doc":"Get the sum of elements in the DDSketch.","ref":"ddskerl_ets.html#sum/1"},{"type":"function","title":"ddskerl_ets.sum/2","doc":"Get the sum of elements in the DDSketch.","ref":"ddskerl_ets.html#sum/2"},{"type":"function","title":"ddskerl_ets.total/1","doc":"Get the total number of elements in the DDSketch.","ref":"ddskerl_ets.html#total/1"},{"type":"function","title":"ddskerl_ets.total/2","doc":"Get the total number of elements in the DDSketch.","ref":"ddskerl_ets.html#total/2"},{"type":"module","title":"ddskerl_exact","doc":"Quantile summary exact implemenation.\n\nThis implements an exact count of all elements seen,\nwhich is the slowest and most memory consuming algorithm.\n\nIt is on the other hand an exact algorithm,\nhence it can be used for comparison against all other approximations.","ref":"ddskerl_exact.html"},{"type":"opaque","title":"ddskerl_exact.ddsketch/0","doc":"DDSketch instance.","ref":"ddskerl_exact.html#t:ddsketch/0"},{"type":"function","title":"ddskerl_exact.insert/2","doc":"Insert a value into the DDSketch.","ref":"ddskerl_exact.html#insert/2"},{"type":"function","title":"ddskerl_exact.merge/2","doc":"Merge two DDSketch instances.","ref":"ddskerl_exact.html#merge/2"},{"type":"function","title":"ddskerl_exact.new/0","doc":"Create a new DDSketch instance.","ref":"ddskerl_exact.html#new/0"},{"type":"function","title":"ddskerl_exact.quantile/2","doc":"Calculate the quantile of a DDSketch.","ref":"ddskerl_exact.html#quantile/2"},{"type":"function","title":"ddskerl_exact.sum/1","doc":"Get the sum of elements in the DDSketch.","ref":"ddskerl_exact.html#sum/1"},{"type":"function","title":"ddskerl_exact.total/1","doc":"Get the total number of elements in the DDSketch.","ref":"ddskerl_exact.html#total/1"},{"type":"module","title":"ddskerl_std","doc":"DDSketch implementation in Erlang.\n\nThis implements an unbounded bucket count, that is, on a degenerate case,\nthe memory consumption will be unbounded logarithmic on the number of different elements.\n\nThe underlying data structure is a map. Note however, that, if measuring picoseconds,\na map with 2184 buckets would keep track of all values all the way to 107 days.\n\nBecause the data structure cannot be accessed concurretly (share-nothing semantics),\na good strategy would be to keep many running in parallel, and running queries over a merge.\n\n> #### When to use {: .tip}\n> This is a good choice when the summary is tracked by a process\n> and updates the state through its mailbox.\n>\n> For example, the following could be used as a template:\n> ```erlang\n> ...\n> -behaviour(gen_server).\n>\n> -spec start_link(ddskerl_std:opts()) -> gen_server:start_ret().\nstart_link(Opts) ->\n>    gen_server:start_link(?MODULE, Opts, [{spawn_opt, [{message_queue_data, off_heap}]}]).\n>\n> -spec init(ddskerl_std:opts()) -> {ok, ddskerl_std:ddsketch()}.\n> init(Opts) ->\n>    {ok, ddskerl_std:new(Opts)}.\n>\n> handle_call({get_quantile, Q}, _From, Sketch) ->\n>    {reply, ddskerl_std:quantile(Sketch, Q), Sketch}.\n>\n> handle_cast({new_value, Value}, Sketch) ->\n>    {noreply, ddskerl_std:insert(Sketch, Value)}.\n> ...\n> ```","ref":"ddskerl_std.html"},{"type":"opaque","title":"ddskerl_std.ddsketch/0","doc":"DDSketch instance.","ref":"ddskerl_std.html#t:ddsketch/0"},{"type":"function","title":"ddskerl_std.insert/2","doc":"Insert a value into the DDSketch.","ref":"ddskerl_std.html#insert/2"},{"type":"function","title":"ddskerl_std.merge/2","doc":"Merge two DDSketch instances.","ref":"ddskerl_std.html#merge/2"},{"type":"function","title":"ddskerl_std.new/1","doc":"Create a new DDSketch instance.","ref":"ddskerl_std.html#new/1"},{"type":"type","title":"ddskerl_std.opts/0","doc":"Options for the DDSketch.","ref":"ddskerl_std.html#t:opts/0"},{"type":"function","title":"ddskerl_std.quantile/2","doc":"Calculate the quantile of a DDSketch.","ref":"ddskerl_std.html#quantile/2"},{"type":"function","title":"ddskerl_std.sum/1","doc":"Get the sum of elements in the DDSketch.","ref":"ddskerl_std.html#sum/1"},{"type":"function","title":"ddskerl_std.total/1","doc":"Get the total number of elements in the DDSketch.","ref":"ddskerl_std.html#total/1"},{"type":"extras","title":"Overview","doc":"[![Hex.pm](https://img.shields.io/hexpm/v/ddskerl.svg)](https://hex.pm/packages/ddskerl)\n[![Hex.pm](https://img.shields.io/hexpm/dt/ddskerl.svg)](https://hex.pm/packages/ddskerl)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/ddskerl/)\n[![GitHub Actions](https://github.com/NelsonVides/ddskerl/actions/workflows/main.yml/badge.svg)](https://github.com/NelsonVides/ddskerl/actions/workflows/main.yml)\n[![Codecov](https://codecov.io/github/NelsonVides/ddskerl/graph/badge.svg?token=G9HB5UKNIY)](https://codecov.io/github/NelsonVides/ddskerl)\n\nThese implementations are all based on the paper\n[\"DDSketch: A Fast and Fully-Mergeable Quantile Sketch with Relative-Error Guarantees\"](https://arxiv.org/pdf/1908.10693)\nby Charles Masson, Jee E. Rim, Homin K. Lee, and Minos N. Garofalakis.","ref":"readme.html"},{"type":"extras","title":"Implementations - Overview","doc":"There are several variations of the implementation, that is:\n- `m:ddskerl_std`\n- `m:ddskerl_bound`\n- `m:ddskerl_counters`\n- `m:ddskerl_ets`\n\nThey differ in their implementation details and their advantages and disadvantages.\nFor details, see the documentation for each module.\n\nGenerally, `m:ddskerl_std` is the most correct and simplest to use, but also the slowest\n(it uses a single map that cannot be shared) and most memory consuming (`log(N)` on the number of\n_different_ elements, which might actually suffice for your purposes). `m:ddskerl_counters` is the\nfastest and most paralellisable, by an order of magnitude, but it requires setting the number of\nbuckets based on the expected highest value (which if measuring picoseconds the largest signed\n64-bits integer would allow you to measure ~107 days, which might actually suffice too).","ref":"readme.html#implementations"},{"type":"extras","title":"API - Overview","doc":"For all merge operations, merges are only defined if the sketches are of the same implementation,\nare defined with the same error (and therefore the same gamma), and in the case of bucketed\nsolutions (counters and ets), the same bucket count.\n- `ddskerl_std:merge/2`,\n- `ddskerl_bound:merge/2`,\n- `ddskerl_counters:merge/2`,\n- `ddskerl_std:merge/2`,\n\nFor all quantile operations, accepted quantiles go from `0.0` (the minimum), to `1.0` (the maximum).\n- `ddskerl_std:quantile/2`,\n- `ddskerl_bound:quantile/2`,\n- `ddskerl_counters:quantile/2`,\n- `ddskerl_std:quantile/2`,","ref":"readme.html#api"},{"type":"extras","title":"License","doc":"# Apache License\nVersion 2.0, January 2004\n\nhttp://www.apache.org/licenses/\n\nTERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION","ref":"license.html"},{"type":"extras","title":"1. Definitions. - License","doc":"\"License\" shall mean the terms and conditions for use, reproduction, and\ndistribution as defined by Sections 1 through 9 of this document.\n\n\"Licensor\" shall mean the copyright owner or entity authorized by the copyright\nowner that is granting the License.\n\n\"Legal Entity\" shall mean the union of the acting entity and all other entities\nthat control, are controlled by, or are under common control with that entity.\nFor the purposes of this definition, \"control\" means (i) the power, direct or\nindirect, to cause the direction or management of such entity, whether by\ncontract or otherwise, or (ii) ownership of fifty percent (50%) or more of the\noutstanding shares, or (iii) beneficial ownership of such entity.\n\n\"You\" (or \"Your\") shall mean an individual or Legal Entity exercising\npermissions granted by this License.\n\n\"Source\" form shall mean the preferred form for making modifications, including\nbut not limited to software source code, documentation source, and configuration\nfiles.\n\n\"Object\" form shall mean any form resulting from mechanical transformation or\ntranslation of a Source form, including but not limited to compiled object code,\ngenerated documentation, and conversions to other media types.\n\n\"Work\" shall mean the work of authorship, whether in Source or Object form, made\navailable under the License, as indicated by a copyright notice that is included\nin or attached to the work (an example is provided in the Appendix below).\n\n\"Derivative Works\" shall mean any work, whether in Source or Object form, that\nis based on (or derived from) the Work and for which the editorial revisions,\nannotations, elaborations, or other modifications represent, as a whole, an\noriginal work of authorship. For the purposes of this License, Derivative Works\nshall not include works that remain separable from, or merely link (or bind by\nname) to the interfaces of, the Work and Derivative Works thereof.\n\n\"Contribution\" shall mean any work of authorship, including the original version\nof the Work and any modifications or additions to that Work or Derivative Works\nthereof, that is intentionally submitted to Licensor for inclusion in the Work\nby the copyright owner or by an individual or Legal Entity authorized to submit\non behalf of the copyright owner. For the purposes of this definition,\n\"submitted\" means any form of electronic, verbal, or written communication sent\nto the Licensor or its representatives, including but not limited to\ncommunication on electronic mailing lists, source code control systems, and\nissue tracking systems that are managed by, or on behalf of, the Licensor for\nthe purpose of discussing and improving the Work, but excluding communication\nthat is conspicuously marked or otherwise designated in writing by the copyright\nowner as \"Not a Contribution.\"\n\n\"Contributor\" shall mean Licensor and any individual or Legal Entity on behalf\nof whom a Contribution has been received by Licensor and subsequently\nincorporated within the Work.","ref":"license.html#1-definitions"},{"type":"extras","title":"2. Grant of Copyright License. - License","doc":"Subject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable copyright license to reproduce, prepare Derivative Works of,\npublicly display, publicly perform, sublicense, and distribute the Work and such\nDerivative Works in Source or Object form.","ref":"license.html#2-grant-of-copyright-license"},{"type":"extras","title":"3. Grant of Patent License. - License","doc":"Subject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable (except as stated in this section) patent license to make, have\nmade, use, offer to sell, sell, import, and otherwise transfer the Work, where\nsuch license applies only to those patent claims licensable by such Contributor\nthat are necessarily infringed by their Contribution(s) alone or by combination\nof their Contribution(s) with the Work to which such Contribution(s) was\nsubmitted. If You institute patent litigation against any entity (including a\ncross-claim or counterclaim in a lawsuit) alleging that the Work or a\nContribution incorporated within the Work constitutes direct or contributory\npatent infringement, then any patent licenses granted to You under this License\nfor that Work shall terminate as of the date such litigation is filed.","ref":"license.html#3-grant-of-patent-license"},{"type":"extras","title":"4. Redistribution. - License","doc":"You may reproduce and distribute copies of the Work or Derivative Works thereof\nin any medium, with or without modifications, and in Source or Object form,\nprovided that You meet the following conditions:\n\n1. You must give any other recipients of the Work or Derivative Works a copy of\n   this License; and\n\n2. You must cause any modified files to carry prominent notices stating that\n   You changed the files; and\n\n3. You must retain, in the Source form of any Derivative Works that You\n   distribute, all copyright, patent, trademark, and attribution notices from\n   the Source form of the Work, excluding those notices that do not pertain to\n   any part of the Derivative Works; and\n\n4. If the Work includes a \"NOTICE\" text file as part of its distribution, then\n   any Derivative Works that You distribute must include a readable copy of the\n   attribution notices contained within such NOTICE file, excluding those\n   notices that do not pertain to any part of the Derivative Works, in at least\n   one of the following places: within a NOTICE text file distributed as part\n   of the Derivative Works; within the Source form or documentation, if\n   provided along with the Derivative Works; or, within a display generated by\n   the Derivative Works, if and wherever such third-party notices normally\n   appear. The contents of the NOTICE file are for informational purposes only\n   and do not modify the License. You may add Your own attribution notices\n   within Derivative Works that You distribute, alongside or as an addendum to\n   the NOTICE text from the Work, provided that such additional attribution\n   notices cannot be construed as modifying the License.\n\nYou may add Your own copyright statement to Your modifications and may provide\nadditional or different license terms and conditions for use, reproduction, or\ndistribution of Your modifications, or for any such Derivative Works as a whole,\nprovided Your use, reproduction, and distribution of the Work otherwise complies\nwith the conditions stated in this License.","ref":"license.html#4-redistribution"},{"type":"extras","title":"5. Submission of Contributions. - License","doc":"Unless You explicitly state otherwise, any Contribution intentionally submitted\nfor inclusion in the Work by You to the Licensor shall be under the terms and\nconditions of this License, without any additional terms or conditions.\nNotwithstanding the above, nothing herein shall supersede or modify the terms of\nany separate license agreement you may have executed with Licensor regarding\nsuch Contributions.","ref":"license.html#5-submission-of-contributions"},{"type":"extras","title":"6. Trademarks. - License","doc":"This License does not grant permission to use the trade names, trademarks,\nservice marks, or product names of the Licensor, except as required for\nreasonable and customary use in describing the origin of the Work and\nreproducing the content of the NOTICE file.","ref":"license.html#6-trademarks"},{"type":"extras","title":"7. Disclaimer of Warranty. - License","doc":"Unless required by applicable law or agreed to in writing, Licensor provides the\nWork (and each Contributor provides its Contributions) on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied,\nincluding, without limitation, any warranties or conditions of TITLE, NON-\nINFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are\nsolely responsible for determining the appropriateness of using or\nredistributing the Work and assume any risks associated with Your exercise of\npermissions under this License.","ref":"license.html#7-disclaimer-of-warranty"},{"type":"extras","title":"8. Limitation of Liability. - License","doc":"In no event and under no legal theory, whether in tort (including negligence),\ncontract, or otherwise, unless required by applicable law (such as deliberate\nand grossly negligent acts) or agreed to in writing, shall any Contributor be\nliable to You for damages, including any direct, indirect, special, incidental,\nor consequential damages of any character arising as a result of this License or\nout of the use or inability to use the Work (including but not limited to\ndamages for loss of goodwill, work stoppage, computer failure or malfunction, or\nany and all other commercial damages or losses), even if such Contributor has\nbeen advised of the possibility of such damages.","ref":"license.html#8-limitation-of-liability"},{"type":"extras","title":"9. Accepting Warranty or Additional Liability. - License","doc":"While redistributing the Work or Derivative Works thereof, You may choose to\noffer, and charge a fee for, acceptance of support, warranty, indemnity, or\nother liability obligations and/or rights consistent with this License. However,\nin accepting such obligations, You may act only on Your own behalf and on Your\nsole responsibility, not on behalf of any other Contributor, and only if You\nagree to indemnify, defend, and hold each Contributor harmless for any liability\nincurred by, or claims asserted against, such Contributor by reason of your\naccepting any such warranty or additional liability.\n\nEND OF TERMS AND CONDITIONS\n\nCopyright 2025, Nelson Vides  .\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\nhttp://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"license.html#9-accepting-warranty-or-additional-liability"}],"proglang":"erlang","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}